

class ColorBinarization : public Tracker {

public:
	ColorBinarization(){
		
		// Hue
		firstRange[0] = 0;
		firstRange[1] = 180;

		// Saturation
		secondRange[0] = 0; 
		secondRange[1] = 256;

		// Ranges
		histRange[0] = firstRange;
		histRange[1] = secondRange;

		channels[0] = 0;
		channels[1] = 1;
		
		histBins[0] = firstBins;
		histBins[1] = secondBins;

	}
	~ColorBinarization(){}

	
	void track(){

		//cvtColor(currentFrame, currentFrame, CV_BGR2GRAY);

		frame.copyTo(currentFrame);
		if (currentFrame.channels() > 1) {
			cvtColor(currentFrame, currentFrameForHist, COLOR_BGR2HSV);	// HSV for Histogram
			

			histRange[0] = firstRange;
			histRange[1] = secondRange;

			binFactor = binFactorColor;

		} else {
			// if the dataset is grayscale
			currentFrameForHist = currentFrame;

			nChannels = 1;
			firstRange[1] = 256;		// hue can be max 180, grayscale is 256 (exclusive)
			histRange[0] = firstRange;
			binFactor = binFactorGrayscale;


			cvtColor(frame, frame, CV_GRAY2BGR);						// Frame should be colored for drawing functions
		}


		frameCount++;
		Log() << endl << endl << "Parsing frame #" << frameCount << endl;

		// Initialization
		if (frameCount == 1) {

			objectROI = firstBB;
			//objectROI = Rect(281, 111, 14, 17);
			//objectROI = Rect(157, 120, 10, 9);					// bicycle patch
			//objectROI = Rect(145,31,16,15);							// fish1 patch
			cout << "Object ROI: " << objectROI << endl;


			updateModel();
			updateModelWithBackground();


			drawRect(frame, objectROI);
			showImage(frame);
			waitKey(1000);

			return;
		}


		calcBackProject(&currentFrameForHist, 1, channels, objectModel, backProjection, histRange);

		// If the histogram is binarized you don't need it
		//equalizeHist(backProjection, backProjection);

		showImage(backProjection,"Binarized");
		waitKey(1);


	}

	void updateModel(){

		Mat currentObject = currentFrameForHist(objectROI);

		// Get the Histogram and normalize it
		calcHist(&currentObject, 1, channels, Mat(),
			objectModel, nChannels, histBins, histRange, true, false);

		normalizeHistogram(objectModel, currentObject.rows*currentObject.cols);

		cout << "Object histogram: " << objectModel.size() << ", depth: " << objectModel.depth() << ", channels: " << objectModel.channels() << endl;
		cout << "Sum value: " << sum(objectModel) << endl;

		showImage(objectModel, "Object Model");
	}


	void updateModelWithBackground(){

		Rect backgroundRect = rectScale(objectROI, 3.0f);
		rectInsideImage(currentFrameForHist, backgroundRect);

		// Rectangular region around the object
		Mat detectionMask = Mat(currentFrameForHist.size(), CV_8U, Scalar::all(0));
		detectionMask(backgroundRect).setTo(Scalar::all(255));
		detectionMask(objectROI).setTo(Scalar::all(0));

		

		Mat backgroundModel;

		// Get the Histogram and normalize it
		calcHist(&currentFrameForHist, 1, channels, detectionMask,
			backgroundModel, nChannels, histBins, histRange, true, false);

		int pixels = (detectionMask.rows * detectionMask.cols) - (objectROI.width * objectROI.height);
		normalizeHistogram(backgroundModel, pixels);
		showImage(backgroundModel, "Background model");

	
		// Object Histogram Binarization
		for (int i = 0; i < objectModel.rows; i++) {
			for (int j = 0; j < objectModel.cols; j++) {

				float obj = objectModel.at<float>(i, j);
				float bg = backgroundModel.at<float>(i, j);

				if (obj>bg*binFactor)
					objectModel.at<float>(i, j) = 255;
				else
					objectModel.at<float>(i, j) = 0;

			}
		}

		cout << "Background model sum: " << sum(backgroundModel) << endl;
		showImage(objectModel, "object model filtered");

	}


	void normalizeHistogram(Mat& hist, int pixels = 0){

		hist /= pixels;
		normalize(hist, hist, 0, 255, NORM_MINMAX, -1, Mat());
		//normalize(hist, hist, 255, 0, NORM_L1);

		//hist /= pixels;
		//hist *= 255;
	}


public:
	Mat 
		currentFrameForHist,
		backProjection;

	

	// Object
	MatND objectModel;


	// Histograms

	float binFactor = 20.0f;			// Filter background colors (a greater factor will select more distinctive object colors)							
	float binFactorGrayscale = 3.0f;	// For single-channel images, binFactor should can equal to ~3.0		
	float binFactorColor = 3.0f;		// For hsv images, binFactor can be equal to ~20.0							
									

	float firstRange[2]; 				// Hue
	float secondRange[2];				// Saturation
	const float* histRange[2];			//= { firstRange, secondRange };

	int channels[2];
	int nChannels = 2;

	int firstBins = 80;					// default: 80
	int secondBins = 90;				// default: 90
	int histBins[2]; 

};
