/*
Copyright  2013 Mario Edoardo Maresca
 
Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:
 
The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.
 
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/
class KeyPointEngine {
public:
	Ptr<FeatureDetector>		detector;
	Ptr<DescriptorExtractor>	extractor;
	Ptr<DescriptorMatcher>		matcher;

	String name;							//> Name of the keypoint-based method (ORB, FREAK, ...) used for debug
	Scalar color;							//> Color used to mark keypoints, used for debug

	float ratioThreshold;					//> Nearest neighbor distance ratio. When == 1.0f the NNDR is disabled
	float minAllowedDistance;				//> Minimum distance to identify duplicate keypoints

	size_t idx;								//> Index of vector<KeyPointEngine> engine

	bool useGrid;							//> When useGrid == true, use a GridAdapterFeatureDetector (with ORB it's advisable)

	KeyPointEngine(){}

	KeyPointEngine( Ptr<FeatureDetector> detector, 	
					Ptr<DescriptorExtractor> extractor,
					Ptr<DescriptorMatcher> matcher, 
					String name, 
					Scalar color, 
					float ratioThreshold = 0,
					bool useGrid = true){

		this->detector			= detector;
		this->extractor			= extractor;
		this->matcher			= matcher;
		this->name				= name;
		this->color				= color;
		this->ratioThreshold	= ratioThreshold;
		this->useGrid			= useGrid;

		
		minAllowedDistance = 0;
		if (name == "ORB" || name == "BRISK")
			minAllowedDistance = 15;
		else if (name == "SURF")
			minAllowedDistance = 0.15f;
		else if (name == "SIFT")
			minAllowedDistance = 30;
		else if (name == "FREAK")
			minAllowedDistance = 3;
		else
			cout << "Warning, minAllowedDistance default value not found for "<<name<<endl;
		
	};
	~KeyPointEngine(){};
};