/*
Copyright  2013 Mario Edoardo Maresca
 
Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:
 
The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.
 
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

/**
cv::Mat cv::getRotationMatrix2D( Point2f center, double angle, double scale )
{
angle *= CV_PI/180;
double alpha = cos(angle)*scale;
double beta = sin(angle)*scale;

Mat M(2, 3, CV_64F);
double* m = (double*)M.data;

m[0] = alpha;
m[1] = beta;
m[2] = (1-alpha)*center.x - beta*center.y;
m[3] = -beta;
m[4] = alpha;
m[5] = beta*center.x + (1-alpha)*center.y;

return M;
}
**/
cv::Mat cv::getRotationMatrix2D(Point2f center, double angle, double scale)
{
	angle *= CV_PI / 180;
	double alpha = cos(angle)*scale;
	double beta = sin(angle)*scale;

	Mat M(2, 3, CV_64F);
	double* m = (double*)M.data;

	m[0] = alpha;
	m[1] = beta;
	m[2] = (1 - alpha)*center.x - beta*center.y;
	m[3] = -beta;
	m[4] = alpha;
	m[5] = beta*center.x + (1 - alpha)*center.y;

	return M;
}

Point2f transformPoint(const Mat& M, const Point2f& p) {
	return Point2f( 
		(float)M.at<double>(0,0)*p.x+ (float)M.at<double>(0,1)*p.y+ (float)M.at<double>(0,2),
		(float)M.at<double>(1,0)*p.x+ (float)M.at<double>(1,1)*p.y+ (float)M.at<double>(1,2)
	);
}


Point2f rotatePoint( const Point2f& p, const Point2f& center, const float& angle ) {
	Mat M = getRotationMatrix2D(center,angle,1);
	return transformPoint(M, p);
}


//> Class Localizer
//> Given a Single DMatch find the center and the orientation of the target object in the query image.
class Localizer {
	public:
		Point2f trainCenter;
		Point2f queryCenter;
		float angle;

		Localizer(const Mat& imgTrain) {
			trainCenter.x = imgTrain.cols/2.f;
			trainCenter.y = imgTrain.rows/2.f;
		}

		//> Project the object's center from the training image to the query image
		void useKeyPointMatch(const KeyPoint& kpTrain, const KeyPoint& kpQuery, float scale = 1.0f ) {
			//> Angle
			angle = kpTrain.angle - kpQuery.angle;

			useKeyPointMatch(kpTrain,kpQuery,angle,scale);
		}

		// If we have already calc the angle
		Point2f useKeyPointMatch(const KeyPoint& kpTrain, const KeyPoint& kpQuery, const float& angle, const float& scale) {
			
			this->angle = angle;

			//> Offset between current training point and training center
			Point2f offset = (trainCenter - kpTrain.pt) * scale;

			//> Find the object's center in the query image with a traslation and a rotation
			queryCenter = kpQuery.pt + offset;
			return (queryCenter = rotatePoint(queryCenter,kpQuery.pt,angle));
		}
};


class LocalizerParams{
	public:
		Point2f center;
		float angle;
		float scale;
		

		LocalizerParams(){}
		~LocalizerParams(){}

};