/*
Copyright  2013 Mario Edoardo Maresca
 
Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:
 
The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.
 
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

//> Class to store keypoints of an image with Multiple engines
class MultiImage {
public:
	Mat image;
	vector<Point2f> corners;
	vector<vector<KeyPoint>> keypoint;
	vector<Mat>	desc;
	vector<vector<DMatch>> goodMatch;
	size_t engines;
	bool isNegative;
	unsigned usage;
	string name;
	vector<vector<Mat>> engineDescSingle;	//> Used for matcher.add()
	int frameAdded;
	bool keyTraining;


	MultiImage(Mat i, size_t engines);
	~MultiImage(){};

	void addFeature(unsigned int engineIndex,vector<KeyPoint> k,Mat d);
	void addGoodMatch(vector<DMatch> m);						//> Add a list of good matches
	void addGoodMatch(size_t engineIndex, DMatch m);			//> Add single good match to engineIndex
	vector<Mat>& getEngineDescriptors(size_t engineIndex); 		//> Used for matcher.add();
	void incrementUsage();										//> Increment weight
	void show(size_t objectIndex);
	void clear();
};


MultiImage::MultiImage(Mat i, size_t e){
	usage=0;
	image=i;
	isNegative = false;
	keyTraining = false;
	frameAdded = 1;

	corners.push_back(cvPoint(0,0));
	corners.push_back(cvPoint( image.cols, 0 ));
	corners.push_back(cvPoint( image.cols,image.rows ));
	corners.push_back(cvPoint( 0, image.rows ));

	this->engines=e;
	this->goodMatch.resize(e);
	this->keypoint.resize(e);
	this->desc.resize(e);
	this->engineDescSingle.resize(e);

}

void MultiImage::incrementUsage() {
	usage++;
}

void MultiImage::addFeature(unsigned int engineIndex,vector<KeyPoint> k,Mat d) {
	keypoint[engineIndex] = k;
	desc[engineIndex] = d;

	engineDescSingle[engineIndex].push_back(d);
}

void MultiImage::addGoodMatch(vector<DMatch> m) {
	goodMatch.push_back(m);
}

void MultiImage::addGoodMatch(size_t engineIndex, DMatch m) {

	goodMatch[engineIndex].push_back(m);

}

void MultiImage::show(size_t objectIndex) {
	string name = "Training #"+toString(objectIndex);

	if (this->isNegative)
		name = "Negative " + name;

	showImage(image,name);

}

vector<Mat>& MultiImage::getEngineDescriptors(size_t engineIndex) {

	return engineDescSingle[engineIndex];
}

void MultiImage::clear() {
	goodMatch.clear();
	this->goodMatch.resize(this->engines);
}
