// Various runner helper function for different datasets/challenges

// Use VTO class to run the VTO benchmark
void runVOT(Ptr<Tracker> tracker) {

	//load region, images and prepare for output
    //	VOT vot_io("region.txt", "images.txt", "output.txt");		// VOT initialization

    trax::Server handle(trax::Metadata(TRAX_REGION_RECTANGLE, TRAX_IMAGE_PATH |
                         TRAX_IMAGE_MEMORY | TRAX_IMAGE_BUFFER), trax_no_log);

	Log::setLevel(0);
	tracker->debug = true;

	//bool hideFrame = fileExists("C:/trackerConfig/hideFrame.txt");
	bool hideFrame = true;
	if (hideFrame)
		tracker->debug = false;

	while (1) {

        trax::Image image;
        trax::Region region;
        trax::Properties properties;

        int tr = handle.wait(image, region, properties);
        if (tr == TRAX_INITIALIZE) {
            // VOT bounding box initialization
            tracker->firstBB = trax::region_to_rect(region);
			tracker->corners = trax::region_to_points(region);

            handle.reply(region, trax::Properties());

        } else if (tr == TRAX_FRAME) {
            tracker->frame = trax::image_to_mat(image);
            tracker->track();

            Rect result = tracker->objectROI;
            handle.reply(trax::rect_to_region(result), trax::Properties());

        } else {
            break;
        }

		// Keyboard shortcut
//		if (!keyboardManagerStandard(1))
//			break;
	}
}

Point webcamPoint1, webcamPoint2;
Rect userRect; 
Mat webcamFrame;
int drag = 0;
void mouseHandler(int event, int x, int y, int flags, void* param) {

	// left button clicked. ROI selection begins
	if (event == CV_EVENT_LBUTTONDOWN && !drag) {
		webcamPoint1 = Point(x, y);
		//cout << "Top-Left point: " << point1<<endl;
		drag = 1;
	}

	// mouse dragged.ROI being selected 
	if (event == CV_EVENT_MOUSEMOVE && drag) {
		Mat img1 = webcamFrame.clone();
		webcamPoint2 = Point(x, y);
		rectangle(img1, webcamPoint1, webcamPoint2, CV_RGB(255, 0, 0), 3, 8, 0);
		imshow("WebcamInit", img1);
	}

	if (event == CV_EVENT_LBUTTONUP && drag) {
		webcamPoint2 = Point(x, y);
		userRect = Rect(webcamPoint1.x, webcamPoint1.y, x - webcamPoint1.x, y - webcamPoint1.y);
		drag = 0;
	}

}

// Run the tracker with the webcam. Let the user select the bb
void runWebcam(Ptr<Tracker> tracker) {

	VideoCapture cap(0); // open the default camera
	if (!cap.isOpened())  // check if we succeeded
		Log::fatal("Webcam could not be opened");

	namedWindow("WebcamInit",CV_WINDOW_NORMAL);
	setMouseCallback("WebcamInit", mouseHandler, NULL);
	bool selected = false;

	CircularBuffer<Point2f> trajectory(60);
	while(true)	{
		cap >> webcamFrame; // get a new frame from camera

		if (drag == 1) {
			cout << "Press any key to continue..." << endl;
			waitKey(0);
			selected = true;
		}
		
		// The user has selected the object. Start tracking...
		if (selected) {
			if (tracker->frameCount == 0) {
				destroyWindow("WebcamInit");
				tracker->firstBB = userRect;
			}

			// Track on the current frame
			tracker->frame = webcamFrame;
			tracker->track();
		
			// Show trajectory (last bounding box centers)
			Point first;
			for (int i = 0; i < trajectory.buffSize-5; ++i){
				Point second = trajectory.get();
				if (first != Point(0, 0) && second != Point(0, 0) )
					line(tracker->frame, first, second, Scalar(0, 0, 255), 3);
				first = second;
			}

			// Show frame
			tracker->showFrame();

			// Append a new point to the trajectory
			trajectory.append(rectCenter(tracker->objectROI));

		} else
			showImage(webcamFrame,"WebcamInit");

		// Keyboard shortcut
		if (!keyboardManagerStandard(1))
			break;
	}

}
