
// Calc the Histogram and normalize it
Mat getHistogram(const Mat& im) {

	// Histogram parameters
	float firstRange[2]		= { 0, 256 }; 				// B
	float secondRange[2]	= { 0, 256 };				// G
	float thirdRange[2]		= { 0, 256 };				// R

	const float* histRange[3] = {firstRange, secondRange, thirdRange};			

	int channels[3] = {0,1,2};							// Channels
	int nChannels = 1;

	int bins = 40;										// Bins
	int histBins[3] = {bins, bins, bins};

	Mat objectHistogram;

	// Get the Histogram and normalize it
	calcHist(&im, 1, channels, Mat(),
		objectHistogram, nChannels, histBins, histRange, true, false);

	int imageArea = im.rows*im.cols;

	//cout << "Image area: "<< imageArea <<", sum: "<< sum(objectHistogram) << endl;

	// Scaling
	objectHistogram = objectHistogram / imageArea;

	return objectHistogram;
}


Mat histogramTemplateMatching(const Mat& modelHistogram, const Mat& frame, Size patchSize) {

	Mat result(frame.rows - patchSize.height, frame.cols - patchSize.width, CV_32F);

	for (auto y = 0; y < result.size().height; ++y){
		for (auto x = 0; x < result.size().width; ++x){

			//cout << "I(" << x << "," << y << ")" << endl;

			Mat patchHistogram = getHistogram(frame(Rect(x,y,patchSize.width,patchSize.height)));

			float r = compareHist(modelHistogram, patchHistogram, CV_COMP_CHISQR);
			//float r = EMD(modelHistogram, patchHistogram,CV_DIST_L1);

			result.at<float>(y, x) = r;
		}
	}

	return result;

}

class HistMatch : public Tracker {

		
public:
	void track(){

		this->prepare();


		if (frameCount == 1) {
			objectROI = firstBB;

			objectHistogram = getHistogram(currentFrame(firstBB));
			objectSize = firstBB.size();

			return;
		}

		//Mat matchingFrame;
		//resize(currentFrame, matchingFrame, Size(), 0.5, 0.5);

		Mat result = histogramTemplateMatching(objectHistogram, currentFrame, objectSize);

		//resize(result, result, Size(), 2.0, 2.0);

		showImage(result, "Histogram matching result");

		Rect searchWindow(0, 0, frame.size().width, frame.size().height);

		//> Localizing the best match with minMaxLoc
		double minVal, maxVal;
		Point minLoc, maxLoc, matchLoc;

		minMaxLoc(result, &minVal, &maxVal, &minLoc, &maxLoc, Mat());
		matchLoc = minLoc;
		cout << "Value: " << maxVal << endl;

		Point topLeft = matchLoc;
		//matchLoc = Point(matchLoc.x + searchWindow.x, matchLoc.y + searchWindow.y);
		objectROI = Rect(matchLoc.x, matchLoc.y, objectSize.width, objectSize.height);
		cout << "Frame: "<<frame.size()<<" Object ROI: " << objectROI << endl;



		isObjectFound = true;

		//drawRect(frame, activeTemplateROI);	//> Draw bounding box
		//showImage(frame, "Tracker (template)");

	}

private:
	Mat objectHistogram;
	Size objectSize;

};